% This function computes the matrices Lambda (LAM), Gamma (GAM), Phi(0)
% (Phi0), Psi(0) (Psi0) as well as the list of unstable eigenvalues with
% multiplicities (lambda) and positive imaginary parts for the linear DDE 
% x'(t)=A0x(t)+A1x(t-tau). It is assumed that for each eigenvalue \xi, the
% dimension of the kernel of the characteristic matrix 
% Delta(\xi)=I\xi - A0 - A1e^{-\xi\tau) is equal to its multiplicity. 
% If this condition is not satisfied, all outputs will be incorrect.

function [LAM,Phi0,Psi0,GAM,eigs_all] = dde_data(A0,A1,tau,N,eig_lower,eig_upper)
n=length(A0);
D=-cheb(N-1)*2/tau;
[Vphi,Jphi]=eig([kron(D(1:N-1,:),eye(n));[A1,zeros(n,(N-2)*n), A0]]);
[Vpsi,Jpsi]=eig(transpose([kron(D(1:N-1,:),eye(n));[A1,zeros(n,(N-2)*n), A0]]));
[~,indphi] = sort(diag(Jphi),'ComparisonMethod','real');
Jphis = Jphi(indphi,indphi);
Vphis = Vphi(:,indphi);
[~,indpsi] = sort(diag(Jpsi),'ComparisonMethod','real');
Jpsis = Jpsi(indpsi,indpsi);
Vpsis = Vpsi(:,indpsi);
lambda_total = [];
LAM = [];
Vphi = [];
Vpsi = [];
eigs_all = [];

if nargin == 4
    eigs_all = diag(Jphis);
    Phi0=[];
    Psi0=[];
    GAM=[];
else
    for i=1:n*N
        eigs_all = [eigs_all;Jphis(i,i)];
        if real(Jphis(i,i))>=eig_lower
            if real(Jphis(i,i))<=eig_upper
                lambda_total = [lambda_total;Jphis(i,i)];
                Vphi = [Vphi,Vphis(:,i)];
                Vpsi = [Vpsi,Vpsis(:,i)];
            end
        end
    end

    lambda_pos = [];
    Vphi_pos = [];
    Vpsi_pos = [];

    for i=1:length(lambda_total)
        if imag(lambda_total(i))>=0
            lambda_pos=[lambda_pos;lambda_total(i)];
            Vphi_pos = [Vphi_pos,Vphi(:,i)];
            Vpsi_pos = [Vpsi_pos,Vpsi(:,i)];
        end
    end  

    m0 = length(lambda_pos);

    for i=1:m0
        if imag(lambda_pos(i))==0
            LAM=blkdiag(LAM,lambda_pos(i));
        else
            LAM = blkdiag(LAM,[real(lambda_pos(i)),-imag(lambda_pos(i));
                imag(lambda_pos(i)),real(lambda_pos(i))]);
        end
    end

    Phi0 = [];
    Psi0 = [];

    for i=1:length(lambda_pos)
        if imag(lambda_pos(i))==0
            Phi0 = [Phi0, Vphi_pos(n*(N-1)+1:n*N,i)];
            Psi0 = [Psi0, Vpsi_pos(n*(N-1)+1:n*N,i)];
        else
            Phi0=[Phi0, [real(Vphi_pos(n*(N-1)+1:n*N,i)),imag(Vphi_pos(n*(N-1)+1:n*N,i))]];
            Psi0=[Psi0, [real(Vpsi_pos(n*(N-1)+1:n*N,i)),imag(Vpsi_pos(n*(N-1)+1:n*N,i))]];
        end
    end

    Psi0 = transpose(Psi0);

    integrandGAM = @(s)expm(-s*LAM)*Psi0*A1*Phi0*expm(LAM*s);

    invGAM = Psi0*Phi0 - integral(integrandGAM,0,tau,'ArrayValued',1);
    GAM = inv(invGAM);
end

end