% Load the workspace for the example.
load workspace_computing.mat
%% User inputs ---
A0 = -R;
A1 = R*E;
tau = 0.2;

% -- build the control basis --
control_array = cell(1,30*29);
for i=1:30
    for j=1:29
        if i+j<=30
            control_array{(i-1)*29 + j}=zeros(30,30);
            control_array{(i-1)*29 + j}(i,i)=-1;
            control_array{(i-1)*29 + j}(i+j,i)=1;
        else
            control_array{(i-1)*29 + j}=zeros(30,30);
            control_array{(i-1)*29 + j}(i,i)=-1;
            control_array{(i-1)*29 + j}(mod(i+j,30),i)=1;
        end
    end
end
control_basis = cell2mat(control_array);
clear control_array
% -- control basis built

h = 10;
gamma = -0.6026;

% -- build the constraint matrix -- 
C = kron(eye(30,30),ones(1,29));
% -- constraint matrix built 

constraint = @(x)[-x ; C*x - ones(30,1)];
cost_weight = 1;
cost_general = [];
searchmode = 'fmincon';
guess = zeros(30*29,1);
options = optimoptions('fmincon','MaxFunctionEvaluations',1E5);
%% Conditioning parameters --
eig_lower = -1.8140;
eig_upper = 0;
spectrum_discretization = 10;
refining = 'false';
%% Script


n = length(A0);
k = (1/n)*size(control_basis)*[0;1];

if strcmp(refining,'true')==0
    [LAM,Phi0,Psi0,GAM,eigs_all] = dde_data(A0,A1,tau,spectrum_discretization,eig_lower,eig_upper);
    if eig_upper == inf
        fprintf('Found %d eigenvalues with real part at least %f for the input DDE, counting multiplicity.\n\n',length(LAM),eig_lower);
    else
        fprintf('Found %d eigenvalues with real part in the interval [%f,%f] for the input DDE, counting multiplicity.\n\n',length(LAM),eig_lower,eig_upper);
    end
    [M0,Z,M0_vectorized,Z_vectorized] = probematrix(LAM,Phi0,Psi0,GAM,h,control_basis);
end 

[x,cost] = optimize(M0_vectorized,Z_vectorized,h,...
    control_basis,constraint,cost_weight,cost_general,gamma,guess,searchmode,options);

control = cell(2);
control{1} = x;
control{2} = zeros(n,n);
for i=1:k
    control{2} = control{2} + x(i)*control_basis(:,(i-1)*n+1:i*n);
end