%  Load the workspace containing the small-world network G, and matrices V and
%  W from the example. Note that A=-laplacian(G).
load workspace_network.mat
%% User inputs ---
A0 = kron(eye(100),-eye(2)+V) + kron(A,eye(2));
A1 = kron(eye(100),W);
tau = 1;

% -- Build the control basis --
row = [[1,0;0,0],[0,0;0,1]];
control_array = cell(1,200);
for i=1:100
    for j=1:2
        control_array{(i-1)*2 + j} = zeros(200,200);
        control_array{(i-1)*2 + j}(2*(i-1)+1:2*i,2*(i-1)+1:2*i) = row(:,2*j-1:2*j);
    end
end
control_basis = cell2mat(control_array);
% -- Control basis built --

h = 10;
gamma = 0;
constraint = @(x)[x;-1-x];
cost_weight = kron( diag(abs(diag(A))) , eye(2) )/max(abs(diag(A)));
cost_general = @(x)sum(tanh(4*abs(x)));
searchmode = 'patternsearch';
guess = -0.5*ones(200,1);
options = [];
%% Conditioning parameters --
eig_lower = -0.5;
eig_upper = inf;
spectrum_discretization = 10;
refining = 'false';

%% Script


n = length(A0);
k = (1/n)*size(control_basis)*[0;1];

if strcmp(refining,'true')==0
    [LAM,Phi0,Psi0,GAM,eigs_all] = dde_data(A0,A1,tau,spectrum_discretization,eig_lower,eig_upper);
    if eig_upper == inf
        fprintf('Found %d eigenvalues with real part at least %f for the input DDE, counting multiplicity.\n\n',length(LAM),eig_lower);
    else
        fprintf('Found %d eigenvalues with real part in the interval [%f,%f] for the input DDE, counting multiplicity.\n\n',length(LAM),eig_lower,eig_upper);
    end
    [M0,Z,M0_vectorized,Z_vectorized] = probematrix(LAM,Phi0,Psi0,GAM,h,control_basis);
end 

[x,cost] = optimize(M0_vectorized,Z_vectorized,h,...
    control_basis,constraint,cost_weight,cost_general,gamma,guess,searchmode,options);

control = cell(2);
control{1} = x;
control{2} = zeros(n,n);
for i=1:k
    control{2} = control{2} + x(i)*control_basis(:,(i-1)*n+1:i*n);
end