function [x,fval]=optimize(M0_vectorized,Z_vectorized,h,basis,constraint,cost1_weight,cost1_general,gamma,guess,searchmode,options)
if nargin == 8
    fprintf('Search mode unspecified; using genetic.\n\n')
    searchmode = 'ga';
    options = [];
end
n=size(basis)*[1;0];
d=sqrt(length(Z_vectorized));
k=(1/n)*size(basis)*[0;1];

if isempty(cost1_general)==1
    costflag = 1;
    cost1_general = @(s)0;
else
    costflag = 0;
    fprintf('General nonlinear cost supplied. Optimization to be completed in control space.\n\n')
end

if isempty(constraint)==1
    constraint = @(s)0;
end


if or( or(rank(M0_vectorized)<d , k<=d^2), costflag==0)
    if rank(M0_vectorized)<d
        fprintf('M0 of deficient rank; unconstrained problem might be infeasible.\n\n');
    end
    if k<=d^2
        fprintf('Dimension of control space (%d) is at most probe space dimension (%d).\n\n',k,d^2);
    end
    fprintf('Performing optimization in control space.\n\n')
    cost = @(x)x.'*cost1_weight*x + cost1_general(x);
    if isempty(guess)==1
        guess = randn(k,1);
    end
    if strcmp(searchmode,'fmincon')==1
        [x,fval]=fmincon(cost,guess,[],[],[],[],[],[],@constraint_fullspace,options);
    elseif strcmp(searchmode,'patternsearch')==1
        [x,fval]=patternsearch(cost,guess,[],[],[],[],[],[],@constraint_fullspace,options);
    elseif strcmp(searchmode,'ga')==1
        [x,fval]=ga(@(x)cost(x'),k,[],[],[],[],[],[],@(x)constraint_fullspace(x'),options);
        x=x';
    end
else
   fprintf('Dimension of control space (%d) is at least probe space dimension (%d). Unconstrained problem is provably feasible. Performing optimization in probe space.\n\n',k,d^2);
   M0plus = pinv(M0_vectorized);
   M0_null = null(M0_vectorized);
   UPCC = @(P) M0plus*(P-Z_vectorized) - M0_null*M0_null.'*cost1_weight*M0plus*(P-Z_vectorized);
   Pcost = @(P) UPCC(P).'*cost1_weight*UPCC(P);
   if isempty(guess)==1
       guess = randn(d^2,1);
   else
       guess = M0_vectorized*guess + Z_vectorized;
   end
   if strcmp(searchmode,'fmincon')==1
       [x,fval]=fmincon(Pcost,guess,[],[],[],[],[],[],@constraint_probe);
   elseif strcmp(searchmode,'patternsearch')==1
       [x,fval]=patternsearch(Pcost,guess,[],[],[],[],[],[],@constraint_probe);
   elseif strcmp(searchmode,'ga')==1
       [x,fval]=ga(@(P)Pcost(P'),d^2,[],[],[],[],[],[],@(x)constraint_probe(x'));
       x=x';
   end
   x=UPCC(x);
end

% -- constraint functions --
    function [c,ceq]=constraint_fullspace(x)
        c = [abs(eigs(reshape(Z_vectorized,[d,d]) + reshape(M0_vectorized*x,[d,d]),1,'largestabs'))-exp(gamma/h);
        constraint(x)];
        ceq = 0;
    end

    function [c,ceq]=constraint_probe(x)
        c = [abs(eigs(reshape(x,[d,d]),1,'largestabs'))-exp(gamma/h);
            constraint(M0plus*(x-Z_vectorized))];
        ceq = 0;
    end

end